/*
This script draws road, from a given point to the cursor.

Usage: [[_x,_y], _dir, _prefix] execVM "scripts\HOMJ_RoadPainter_DrawRoad.sqf"

Global variables used:
**HOMJ_RoadPainter_Sections = contains the names of all of the section arrays so they can be referenced[section name, section name]
**HOMJ_RoadPainter_[section name] = contains all of the road piece arrays for the section
**HOMJ_RoadPainter_RoadsTemp = output of this script... an array containing piece arrays, which look like this:
[ type, prefix, dimensions, suffix, start position/dir, end position/dir, object name]
["|", "ca\roads2\asf1_",  [6.25], "", [[x, y], dir], [[x, y], dir], _road];
["+", "ca\roads2\asf1_", [6.25], "_crosswalk", [[x, y], dir], [[x, y], dir], _road];
[")", "ca\roads2\asf1_", [10, 25], "", [[x, y], dir],  [[x, y], dir], _road];

Local variables used:
_HOMJ_RoadPainter_RoadsTemp = built up temp array. This array is compared to the existing global array as it is built, in a way to spawn road pieces for the preview.

e.g.:

temp = [1, 2, 3] -- existing road peices 
_temp = [1, 2] -- new calculated road pieces: if _temp is shorter, then script will delete all excess. then it will perform comparisons like below

temp = [1, 2] -- existing road peices 
_temp = [1, b, c] -- new calculated road pieces: first one is the same, then 2nd one is different, so script will delete and re-make this piece and ones after it

temp = [] -- existing road peices 
_temp = [1, b, c] -- new calculated road pieces: since none are made, script will spawn them

Curves have two dimensions, straights have one dimension. When the tool exports, it will use floor(dimension) to create full road names, e.g. ") ca\roads2\asf1_10 25" ... or "+ ca\roads2\asf1_6_crosswalk"
*/

if (HOMJ_RoadPainter_Painting) exitWith {["ERROR: ALREADY DRAWING ROAD!"] execVM "scripts\HOMJ_RoadPainter_ThrowError.sqf";};
["DRAW YOUR ROAD"] execVM "scripts\HOMJ_RoadPainter_ThrowMessage.sqf";

private ["_rad", "_CalcRoad", "_doLoop", "_LR", "_HOMJ_RoadPainter_RoadsTemp", "_xx", "_yy", "_dirOld", "_dirNew", "_brace", "_start", "_dir"];

_startPos = _this select 0;
_startDir = _this select 1;
_prefix = _this select 2;
_sectionName = _this select 3;

HOMJ_RoadPainter_SectionCurrent = _sectionName;

#define __Rad1 25
#define __Rad2 50
#define __Rad3 75
#define __Rad4 100

#define __turn 10

waitUntil {!isNil "HOMJ_WorldPos"};

RoadCalculating = true;

HOMJ_RoadPainter_Stop = false;
HOMJ_RoadPainter_Painting = true;

HOMJ_RoadPainter_RoadsTemp = []; // Don't confuse this with the LOCAL version of the same variable -- does this variable need to be global??
while {RoadCalculating AND !(HOMJ_RoadPainter_Stop)} do {
	_HOMJ_RoadPainter_RoadsTemp = [];
	
	// ======================= DETERMINE WHETHER OR NOT MOUSE IS IN A VALID LOCATION TO DRAW ROADS:
	
	_endPos = HOMJ_WorldPos; // cursor position
	_tempDist = _endPos distance _startPos;	// Calculate distance between cursor and start pos
	
	// Calculate relative direction between original heading of first point, to second point
	_dirtoPos = [_startPos, _endPos] call dirToPos;
	_dDir = _dirtoPos - _startDir;
	if (_dDir > 180) then {_dDir = _dDir - 360;};
	if (_dDir < -180) then {_dDir = _dDir + 360;};
	
	if (_tempDist > 3) then {
		// calculate relative position for circle-algorithm -- i.e. X,Y coordinates in coordinate space of original point's heading
		_X = _tempDist*sin(_dDir);
		_Y = _tempDist*cos(_dDir);
		
		if (HOMJ_RoadPainter_AutoRad) then {
			// calculate radius of road to use Y + (X-10) = 10
			if (((_Y^2) + (abs(_X) - __Rad1)^2) < (__Rad1-5)^2) then { // within smallest circle's radius; no point in even trying.
				_CalcRoad = true;
				_rad = __Rad1;
			} else {
				_CalcRoad = true;
				if ((((_Y^2) + (abs(_X) - __Rad4)^2) > __Rad4^2) AND (_tempDist> (3.1415*__Rad4/18))) then { // outside of largest circle's radius; use largest radius
					_rad = __Rad4;
				} else {
					if ((((_Y^2) + (abs(_X) - __Rad3)^2) > __Rad3^2) AND (_tempDist> (3.1415*__Rad3/18))) then { // outside of 2nd largest circle's radius
						_rad = __Rad3;
					} else {
						if ((((_Y^2) + (abs(_X) - __Rad2)^2) > __Rad2^2) AND (_tempDist> (3.1415*__Rad2/18))) then { // outside of 3rd largest circle's radius
							_rad = __Rad2;
						} else { // outside of smallest circle's radius
							_rad = __Rad1;
						};
					};
				};
			};
			HOMJ_RoadPainter_ManRad = _rad;
		} else {
			_CalcRoad = true;
			_rad = HOMJ_RoadPainter_ManRad;
		};
	} else {
		_CalcRoad = false;
	};
	
	if (_CalcRoad) then {
				
		((findDisplay 19860) displayCtrl 19861) ctrlSetText format ["Radius: %1m", _rad]; 

		_xx = _startPos select 0;
		_yy = _startPos select 1;

		// Place curved sections until you find minimum dDir *and* distance to target:
		_DirOld = _startDir;

		_dDirMin = _dDir;
		_dDirNew = 0;
		
		_distOld = [_xx, _yy] distance _endPos;
		
		if (_dDir < 0) then {_LR = -1;_brace = ")";} else {_LR = 1;_brace = "(";};
		
		_doLoop = true;
		
		while {_doLoop} do {
			// Calculate where a curved piece will go to
			_dirNew = _dirOld + _LR*__turn; // New tangential heading. Multiply by +- 1 for left/right. Roads always turn in increments of 10 degrees.
			if (_dirNew >= 360) then {_dirNew = _dirNew - 360};
			if (_dirNew < 0) then {_dirNew = _dirNew + 360};
			
			_xComp = _rad - _rad * cos(__turn);
			_yComp = _rad * sin(__turn);
			
			_xxTrial = _xx + _LR*(_xComp * cos(_dirOld)) + (_yComp * sin(_dirOld)); // New world X position - needs to go through 2 coordinate transforms
			_yyTrial = _yy + (_yComp * cos(_dirOld)) - _LR*(_xComp * sin(_dirOld)); // New world y position - needs to go through 2 coordinate transforms
			_dirToPos = [[_xxTrial, _yyTrial], _endPos] call dirToPos; // New DirToPos after adding a turn piece!
			
			_dDirNew = _dirtoPos - _DirNew;
			
			if (_dDirNew > 180) then {_dDirNew = _dDirNew - 360;}; // I now have a dDir from the new position - must check if it's closer than the old dDir
			if (_dDirNew < -180) then {_dDirNew = _dDirNew + 360;}; // I now have a dDir from the new position - must check if it's closer than the old dDir
			_distNew = [_xxTrial, _yyTrial] distance _endPos;
			
			// Try placing a straight road to see if it gets closer than the curve
			_xxTrialS = _xx + (rad(__turn)*_rad*sin(_DirOld));
			_yyTrialS = _yy + (rad(__turn)*_rad*cos(_DirOld));
			_distNewS = [_xxTrialS, _yyTrialS] distance _endPos;

			// Check which is closer to the endpoint, a curve or a straight?
			if ((_distNew < _distNewS) AND (_distNew < _distOld)) then { // ISSUE: for 180 degree turns distNew will never be < _distOld...
				_dDirMin = _dDirNew;
				_RoadPiece = [_brace, _prefix, [__turn, _rad], "", [[_xx, _yy], _dirOld], [[_xxTrial, _yyTrial], _dirNew]];
				_HOMJ_RoadPainter_RoadsTemp = _HOMJ_RoadPainter_RoadsTemp + [_RoadPiece];
				_xx = _xxTrial;
				_yy = _yyTrial;
				_DirOld = _DirNew;
				_distOld = _distNew;
			} else {
				_doLoop = false; // Move on to drawing straights
			};
		};
		
		// Place straight sections until you find minimum distance to target:
		// [ type, prefix, dimensions, suffix, start position/dir, end position/dir, object name]

		_distOld = [_xx, _yy] distance _endPos;
		_doLoop = true;		
		while {_doLoop} do {
			_xxTrial = _xx + (6.25*sin(_DirOld));
			_yyTrial = _yy + (6.25*cos(_DirOld));
			
			_distNew = [_xxTrial, _yyTrial] distance _endPos;
			if ((_distNew < _distOld) AND (([_xxTrial, _yyTrial] distance _startPos) < 500)) then {
				_RoadPiece = ["|", _prefix, [0, 6.25], "", [[_xx, _yy], _dirOld], [[_xxTrial, _yyTrial], _dirOld]];
				_HOMJ_RoadPainter_RoadsTemp = _HOMJ_RoadPainter_RoadsTemp + [_RoadPiece];
				_xx = _xxTrial;
				_yy = _yyTrial;
				_distOld = _distNew;
			} else {
				_doLoop = false; // This is as close to the cursor as the road can get
			};
		};

		// Place small yellow markers to show tangential direction of last placed piece for 30m
		_str = 0;
		_xxTemp = _xx;
		_yyTemp = _yy;
		while {_str < 30} do {
			_str = _str + 1;
			_xxTemp = _xxTemp + (sin(_dirOld));
			_yyTemp = _yyTemp + (cos(_dirOld));
			drop ["\ca\data\cl_basic", "", "Billboard", 1, 0.1, [_xxTemp, _yyTemp, 1], [0,0,0], 0, 0.005, 0.0042, 0, [0.5], [[1,1,0,0.75]], [0,1,0,1,0,1], 0, 0, "", "", ""];
		};
	};
	// I now have an array, _HOMJ_RoadPainter_RoadsTemp, which reflects exactly what needs to be drawn. to avoid as much createVehicle'ing as possible, I will maintain another array which reflects currently drawn pieces.
	// This script will now compare these two arrays, finding where they diverge, and then updating the existing array as required
	
	// Cut amount of drawn roads down to the amount of calculated roads, if needed:
	if ((count _HOMJ_RoadPainter_RoadsTemp) < (count HOMJ_RoadPainter_RoadsTemp)) then { 
		for "_i" from (count _HOMJ_RoadPainter_RoadsTemp) to (count HOMJ_RoadPainter_RoadsTemp - 1) do {
			deleteVehicle ((HOMJ_RoadPainter_RoadsTemp select _i) select 6); // [ type, prefix, dimensions, suffix, start position/dir, end position/dir, *object name*]
			HOMJ_RoadPainter_RoadsTemp set [_i, "DELETE"]; 
		};
		HOMJ_RoadPainter_RoadsTemp = HOMJ_RoadPainter_RoadsTemp - ["DELETE"];
	};

	// Compare calculated road to drawn road:
	for "_i" from 0 to (count _HOMJ_RoadPainter_RoadsTemp - 1) do {
		if (_i <= (count HOMJ_RoadPainter_RoadsTemp - 1)) then { // A road piece already exists, see if it is still correct:
			if (str((_HOMJ_RoadPainter_RoadsTemp select _i) select 5) != str((HOMJ_RoadPainter_RoadsTemp select _i) select 5)) then {
				
				// Delete the old, wrong road:
				deleteVehicle ((HOMJ_RoadPainter_RoadsTemp select _i) select 6); // [ type, prefix, dimensions, suffix, start position/dir, end position/dir, *object name*] THIS IS RoadsTemp select _i
				
				// Update the array entry:
				HOMJ_RoadPainter_RoadsTemp set [_i, (_HOMJ_RoadPainter_RoadsTemp select _i)]; 				
				
				// Spawn a new road piece: Pull the road style out of the prefix, i.e. ... ca\roads2\asf1_ ... changes to asf1_
				_mapfactPrefix = ["RIGHT", "RIGHT", FALSE, ((HOMJ_RoadPainter_RoadsTemp select _i) select 1)] call HOMJ_CullBackslash;
				

				_dims = (HOMJ_RoadPainter_RoadsTemp select _i) select 2;
				if ((_dims select 0) == 0) then { // straight - needs only a length
					_dims = str(floor(_dims select 1));
				} else { // curve - needs degrees and radius
					_dims = format ["%1_%2", floor(_dims select 0), floor(_dims select 1)];
				};
				_roadName = format ["HOMJ_%1%2", _mapfactPrefix, _dims];
				
				_road = createVehicle [_roadName, [0,0,0], [], 0, "NONE"];
				if (((HOMJ_RoadPainter_RoadsTemp select _i) select 0) == ")") then {
					_road setDir ((((HOMJ_RoadPainter_RoadsTemp select _i) select 5) select 1) + 180);
					_road setPos (((HOMJ_RoadPainter_RoadsTemp select _i) select 5) select 0);
				} else {
					_road setDir (((HOMJ_RoadPainter_RoadsTemp select _i) select 4) select 1);
					_road setPos (((HOMJ_RoadPainter_RoadsTemp select _i) select 4) select 0);
				};
				(HOMJ_RoadPainter_RoadsTemp select _i) set [6, _road];
			}; // else road is correct and do nothing
		} else { // Road was not in the drawn array - add it
			// Update the array entry:
			HOMJ_RoadPainter_RoadsTemp set [_i, (_HOMJ_RoadPainter_RoadsTemp select _i)]; 	
			// Spawn a new road piece:
		
			// Pull the road style out of the prefix, i.e. ... ca\roads2\asf1_ ... changes to asf1_
			
			_mapfactPrefix = ["RIGHT", "RIGHT", FALSE, ((HOMJ_RoadPainter_RoadsTemp select _i) select 1)] call HOMJ_CullBackslash;

			_dims = (HOMJ_RoadPainter_RoadsTemp select _i) select 2;

			if ((_dims select 0) == 0) then { // straight - needs only a length
				_dims = str(floor(_dims select 1));
			} else { // curve - needs degrees and radius
				_dims = format ["%1_%2", floor(_dims select 0), floor(_dims select 1)];
			};
			_roadName = format ["HOMJ_%1%2", _mapfactPrefix, _dims];
			_road = createVehicle [_roadName, [0,0,0], [], 0, "NONE"];
			
			if (((HOMJ_RoadPainter_RoadsTemp select _i) select 0) == ")") then {
				_road setDir ((((HOMJ_RoadPainter_RoadsTemp select _i) select 5) select 1) + 180);
				_road setPos (((HOMJ_RoadPainter_RoadsTemp select _i) select 5) select 0);
			} else {
				_road setDir (((HOMJ_RoadPainter_RoadsTemp select _i) select 4) select 1);
				_road setPos (((HOMJ_RoadPainter_RoadsTemp select _i) select 4) select 0);
			};
			(HOMJ_RoadPainter_RoadsTemp select _i) set [6, _road];
		};
	};
	sleep 0.05;

};
HOMJ_RoadPainter_Painting = false;
if !(HOMJ_RoadPainter_Stop) then {
	private ["_finalRoad", "_sectionCount"];
	if !(HOMJ_RoadPainter_Delete) then {
		call compile format ["%1 = %1 + HOMJ_RoadPainter_RoadsTemp;", _sectionName];
	} else {
		{ deleteVehicle (_x select 6); } forEach (HOMJ_RoadPainter_RoadsTemp);
		HOMJ_RoadPainter_Delete = false;
	};
	
	call compile format ["_sectionCount = count %1", _sectionName];
	if (_sectionCount > 0) then {
		call compile format ["_finalRoad = %1 select (count %1 - 1);", _sectionName];
		_startPos = ((_finalRoad select 5) select 0);
		_startDir = ((_finalRoad select 5) select 1);
		[_startPos, _startDir, _prefix, _sectionName] execVM "scripts\HOMJ_RoadPainter_DrawRoad.sqf";
	} else {
		[HOMJ_RoadPainter_SectionStartSelect select 0, HOMJ_RoadPainter_SectionStartSelect select 1, HOMJ_RoadPainter_SectionStartSelect select 2, _sectionName] execVM "scripts\HOMJ_RoadPainter_DrawRoad.sqf";
	};
} else {
	HOMJ_RoadPainter_Stop = false;
	{ deleteVehicle (_x select 6); } forEach (HOMJ_RoadPainter_RoadsTemp);
	[""] execVM "scripts\HOMJ_RoadPainter_ThrowMessage.sqf";
};